      $SET DIRECTIVE "-out:cobolinjava"
      $SET DIRECTIVE "-sql:jdbc     4"
      $SET DIRECTIVE "-sql:user     admin"
      $SET DIRECTIVE "-sql:password mark"
      $SET DIRECTIVE "-sql:driver   org.apache.derby.jdbc.ClientDriver"
      $SET DIRECTIVE "-sql:url      jdbc:derby://localhost:1527/derbyDB"
       IDENTIFICATION DIVISION.
       PROGRAM-ID.    ExecNames.
      *
      * invoke a stored procedure within Apache DB that
      * returns 2 result sets
      *
      * test setup:  install ADMIN.GETNAMES stored procedure
      * into Apache DB (Derby) as indicated in GETNAMES.CBL.
      *
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       INPUT-OUTPUT SECTION.       
       DATA DIVISION.
       
       WORKING-STORAGE SECTION.
       01  SQLREC.
               02  BADMSG    PIC X(34) VALUE
                     ' SQL CALL FAILED DUE TO SQLCODE = '.
               02  BADCODE   PIC +9(5) USAGE DISPLAY.
               02  FILLER    PIC X(80) VALUE SPACES.
       01  ERRMREC.
               02  ERRMMSG   PIC X(12) VALUE ' SQLERRMC = '.
               02  ERRMCODE  PIC X(70).
               02  FILLER    PIC X(38) VALUE SPACES.
       01  CALLREC.
               02  CALLMSG   PIC X(28) VALUE
                     ' GETPRML FAILED DUE TO RC = '.
               02  CALLCODE  PIC +9(5) USAGE DISPLAY.
               02  FILLER    PIC X(42) VALUE SPACES.
       01  RSLTREC.
               02  RSLTMSG   PIC X(15) VALUE
                     ' TABLE NAME IS '.
               02  TBLNAME   PIC X(18) VALUE SPACES.
               02  FILLER    PIC X(87) VALUE SPACES.

       01  WS-NAME            PIC X(10).
       01  WS-OUT-CODE        PIC S9(6).
       01  WS-RESULT          PIC X(10).
       01  I                  PIC S9(9).

      * DECLARE A RESULT SET LOCATOR FOR THE RESULT SET   *
      * THAT IS RETURNED.                                 *
       01  LOC1               USAGE SQL TYPE IS
                              RESULT-SET-LOCATOR VARYING.
       01  LOC2               USAGE SQL TYPE IS
                              RESULT-SET-LOCATOR VARYING.
       
           EXEC SQL INCLUDE SQLCA  END-EXEC.
       
       PROCEDURE DIVISION.
      *------------------
       PROG-START.
      *     Connect to the DB (make sure derbyclient.jar is on
      *     the classpath)
      *     connect to the DB.  Alternative to CONNECT statement
      *     is to start the program with 
      *     -Dsql.default.url=www,      -Dsql.default.user=xxx,
      *     -Dsql.default.password=yyy, -Dsql.default.driver=zzz
            EXEC SQL
                CONNECT TO 'jdbc:derby://localhost:1527/derbyDB'
                USER 'admin'
                PASSWORD 'mark'
                DRIVER 'org.apache.derby.jdbc.ClientDriver'
            END-EXEC.
            IF SQLCODE NOT EQUAL 0 THEN
                DISPLAY "KNOWN - can't connect" UPON SYSOUT
                DISPLAY "SQLCODE " SQLCODE UPON SYSOUT
                DISPLAY "SQLERRMC " SQLERRMC UPON SYSOUT
                GOBACK
            END-IF.
            
      *     set key to middle of the names
            MOVE 'gggggggggg' TO WS-NAME.

      *     invoke the SP which returns two result sets,
      *     tuples before the WS-NAME key and those after it
            EXEC SQL
               CALL GETNAMES(
                          :WS-NAME,
                          :WS-OUT-CODE)
            END-EXEC.

      *     Note:  IBM DB2 returns +466 for "OK", Derby 0
            IF SQLCODE not equal to 0 THEN
                MOVE SQLCODE TO BADCODE
                DISPLAY 'CALL GETNAMES() FAILED WITH SQLCODE '
                    BADCODE " " SQLERRMC UPON SYSOUT
                display "known" upon sysout
                goback
            ELSE
                PERFORM GET-PARMS
                PERFORM GET-RESULT-SET
            END-IF.

       PROG-END.
            display "pass" upon sysout.
            GOBACK.

       GET-PARMS.
      *     verify the call worked
            IF WS-OUT-CODE NOT EQUAL TO 22 THEN
                MOVE WS-OUT-CODE TO BADCODE
                DISPLAY 'CALL GETNAMES() FAILED WITH OUT-CODE '
                    BADCODE UPON SYSOUT
            ELSE
                DISPLAY 'CALL GETNAMES() PASSED WITH OUT-CODE ' 
                    WS-OUT-CODE UPON SYSOUT
            END-IF.

       GET-RESULT-SET.
      *    Associate "locator" variables with the stored proc
           EXEC SQL 
               ASSOCIATE LOCATORS (:LOC1, :LOC2) 
               WITH PROCEDURE GETNAMES
           END-EXEC.
      *    Link the ResultSet to the Locator
           EXEC SQL 
               ALLOCATE C1 CURSOR FOR RESULT SET :LOC1
           END-EXEC.
           IF SQLCODE NOT EQUAL TO 0 THEN
               DISPLAY "FAIL TO ALLOCATE 1st CURSOR" UPON SYSOUT
               GOBACK
           END-IF.
      *
           DISPLAY "" UPON SYSOUT.
           DISPLAY "RECORDS BEFORE '" WS-NAME "':" UPON SYSOUT.
           PERFORM GET-ROWS VARYING I
               FROM 1 BY 1 UNTIL SQLCODE EQUAL TO +100.
      *
           EXEC SQL
               CLOSE C1
           END-EXEC.
           IF SQLCODE NOT EQUAL TO 0 THEN
               DISPLAY "FAIL TO CLOSE 1st CURSOR" UPON SYSOUT
               GOBACK
           END-IF

      *    Link the ResultSet to the Locator (note cursor reuse)
           EXEC SQL 
               ALLOCATE C1 CURSOR FOR RESULT SET :LOC2
           END-EXEC.
           IF SQLCODE NOT EQUAL TO 0 THEN
               DISPLAY "FAIL TO ALLOCATE 2nd CURSOR" UPON SYSOUT
               GOBACK
           END-IF
      *
           DISPLAY "" UPON SYSOUT.
           DISPLAY "RECORDS AFTER '" WS-NAME "':" UPON SYSOUT.
           PERFORM GET-ROWS VARYING I
               FROM 1 BY 1 UNTIL SQLCODE EQUAL TO +100.
      *
           EXEC SQL
               CLOSE C1
           END-EXEC.
           IF SQLCODE NOT EQUAL TO 0 THEN
               DISPLAY "FAIL TO CLOSE 2nd CURSOR" UPON SYSOUT
               GOBACK
           END-IF

       GET-ROWS.
           EXEC SQL 
               FETCH C1 INTO :WS-RESULT
           END-EXEC.
           if sqlcode equal to 0 then
               DISPLAY "  " WS-RESULT UPON SYSOUT
           ELSE IF SQLCODE NOT EQUAL TO +100 THEN
               DISPLAY "  ** ERROR SQLCODE: " SQLCODE UPON SYSOUT
               GOBACK
           END-IF END-IF.
           .