      $SET DIRECTIVE "-out:cobolinjava"
      $SET DIRECTIVE "-sql:jdbc     4"
      $SET DIRECTIVE "-sql:sp       splice"
      *SET DIRECTIVE "-sql:spname   getnames"
      *SET DIRECTIVE "-sql:spschema admin"
      $SET DIRECTIVE "-sql:user     admin"
      $SET DIRECTIVE "-sql:password mark"
      $SET DIRECTIVE "-sql:driver   org.apache.derby.jdbc.ClientDriver"
      $SET DIRECTIVE "-sql:url      jdbc:derby://localhost:1527/derbyDB"
       IDENTIFICATION DIVISION.
       PROGRAM-ID. GETNAMES.
      *
      * the GETNAMES stored procedure
      * implements the stored procedure invoked from Names.cbl
      *            EXEC SQL
      *                CALL GETNAMES(
      *                        :STARTNM,
      *                        :OUT-CODE)
      *            END-EXEC.
      * which returns 2 result sets of single column strings.
      *
      * this stored procedure is declared in Apache DB with
      *
      *       CREATE PROCEDURE GETNAMES(
      *          IN  STARTNM CHAR(10),
      *          OUT OUT_CODE INTEGER)
      *       PARAMETER STYLE JAVA
      *       LANGUAGE JAVA
      *       READS SQL DATA
      *       DYNAMIC RESULT SETS 2
      *       EXTERNAL NAME 'getnames.callSp'
      *
      * returns 2 result sets containing single-column strings based on
      * a database table 'names' defined with this DDL and populated:
      *
      *       CREATE TABLE NAMES(NAME CHAR(10));
      *       INSERT INTO NAMES VALUES('fred');
      *       INSERT INTO NAMES VALUES('barney');
      *       INSERT INTO NAMES VALUES('wilma');
      *       INSERT INTO NAMES VALUES('betty');
      *
      * the first result set returns the names before STARTNM and
      * the second result set returns the names after STARTNM.
      * any SQLCODE error is returned in OUT-CODE.
      *
      * install this stored procedure into the Apache DB (Derby) with the commands
      * (assuming logged in as user 'ADMIN' such that the schema is 'ADMIN'):
      *
      *       CALL SQLJ.remove_jar
      *          ('ADMIN.GETNAMES', 0);
      *       CALL SQLJ.install_jar
      *          ('/data/story812_2.jar', 'ADMIN.GETNAMES', 0);
      *       CALL SYSCS_UTIL.SYSCS_SET_DATABASE_PROPERTY
      *        ('derby.database.classpath', 'ADMIN.GETNAMES');
      *
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       DATA DIVISION.
       WORKING-STORAGE SECTION.
           EXEC SQL
             INCLUDE SQLCA
           END-EXEC.

       01 WS-GEN-VAR.
          05 WS-KEY                         PIC X(10).
          05 WS-ERROR-MSG                   PIC X(32).

           EXEC SQL
              DECLARE BEFORE-CUR CURSOR WITH RETURN FOR
              SELECT NAME
              FROM NAMES
              WHERE NAME <  :WS-KEY
              ORDER BY NAME DESC
           END-EXEC.

      *
           EXEC SQL
              DECLARE AFTER-CUR CURSOR WITH RETURN FOR
              SELECT NAME
              FROM NAMES
              WHERE NAME > :WS-KEY
              ORDER BY NAME ASC
           END-EXEC.

       LINKAGE SECTION.
       01 LK-STARTNM                        PIC X(10).
       01 LK-OUT-CODE                       PIC -9(6) COMP.

       PROCEDURE DIVISION
             USING LK-STARTNM
                  ,LK-OUT-CODE
                  .
      *    set the key for the two queries
           MOVE LK-STARTNM TO WS-KEY.

      *    open the first cursor, returning everything before the key
           EXEC SQL
              OPEN BEFORE-CUR
           END-EXEC
           IF SQLCODE NOT EQUAL 0 THEN
               DISPLAY "OPEN CURSOR 1 SQLCODE: " SQLCODE UPON SYSOUT
           END-IF.

      *    open the second cursor, returning everything after the key
           EXEC SQL
              OPEN AFTER-CUR
           END-EXEC
           IF SQLCODE NOT EQUAL 0 THEN
               DISPLAY "OPEN CURSOR 2 SQLCODE: " SQLCODE UPON SYSOUT
           END-IF.

      *    move the results of the last OPEN to OUT-CODE
           MOVE SQLCODE TO LK-OUT-CODE.

      *    better test ... something other than 0 to OUT-CODE
           MOVE 22 TO LK-OUT-CODE.

      *    return the 2 result sets
           GOBACK
           .
       END PROGRAM GETNAMES.